@@ -107,6 +107,9 @@ class Build(properties.PropertiesMixin):
         self._locks_released = False
         self._build_finished = False
 
+        # tracks the config version for locks
+        self.config_version = None
+
     def setBuilder(self, builder):
         """
         Set the given builder as our builder.
@@ -115,11 +118,12 @@ class Build(properties.PropertiesMixin):
         """
         self.builder = builder
         self.master = builder.master
+        self.config_version = builder.config_version
 
+    @defer.inlineCallbacks
     def setLocks(self, lockList):
-        # convert all locks into their real forms
-        self.locks = [(self.builder.botmaster.getLockFromLockAccess(access), access)
-                      for access in lockList]
+        self.locks = yield self.builder.botmaster.getLockFromLockAccesses(lockList,
+                                                                          self.config_version)
 
     def setWorkerEnvironment(self, env):
         # TODO: remove once we don't have anything depending on this method or attribute
@@ -154,12 +158,12 @@ class Build(properties.PropertiesMixin):
         return files
 
     def __repr__(self):
-        return "<Build %s number:%r results:%s>" % (
-            self.builder.name, self.number, statusToString(self.results))
+        return "<Build {} number:{} results:{}>".format(self.builder.name, repr(self.number),
+                                                        statusToString(self.results))
 
     def blamelist(self):
-        # Note that this algorithm is also implemented in buildbot.reporters.utils.getResponsibleUsersForBuild,
-        # but using the data api.
+        # Note that this algorithm is also implemented in
+        # buildbot.reporters.utils.getResponsibleUsersForBuild, but using the data api.
         # it is important for the UI to have the blamelist easily available.
         # The best way is to make sure the owners property is set to full blamelist
         blamelist = []
@@ -288,7 +292,7 @@ class Build(properties.PropertiesMixin):
 
         worker = workerforbuilder.worker
 
-        log.msg("%s.startBuild" % self)
+        log.msg("{}.startBuild".format(self))
 
         self.build_status = build_status
         # TODO: this will go away when build collapsing is implemented; until
@@ -306,7 +310,8 @@ class Build(properties.PropertiesMixin):
                                                                       str(self.buildid),
                                                                       "stop"))
 
-        # the preparation step counts the time needed for preparing the worker and getting the locks.
+        # the preparation step counts the time needed for preparing the worker and getting the
+        # locks.
         # we cannot use a real step as we don't have a worker yet.
         self.preparation_step = buildstep.BuildStep(name="worker_preparation")
         self.preparation_step.setBuild(self)
@@ -349,7 +354,8 @@ class Build(properties.PropertiesMixin):
             yield self.buildPreparationFailure(ready_or_failure, "worker_prepare")
             if self.stopped:
                 self.buildFinished(["worker", "cancelled"], self.results)
-            elif isinstance(ready_or_failure, Failure) and ready_or_failure.check(interfaces.LatentWorkerCannotSubstantiate):
+            elif isinstance(ready_or_failure, Failure) and \
+                    ready_or_failure.check(interfaces.LatentWorkerCannotSubstantiate):
                 self.buildFinished(["worker", "cannot", "substantiate"], EXCEPTION)
             else:
                 self.buildFinished(["worker", "not", "available"], RETRY)
@@ -365,8 +371,7 @@ class Build(properties.PropertiesMixin):
         # us in a build.
         yield self.master.data.updates.setBuildStateString(self.buildid,
                                                            'pinging worker')
-        log.msg("starting build %s.. pinging the worker %s"
-                % (self, workerforbuilder))
+        log.msg("starting build {}.. pinging the worker {}".format(self, workerforbuilder))
         try:
             ping_success_or_failure = yield workerforbuilder.ping()
         except Exception:
@@ -433,10 +438,10 @@ class Build(properties.PropertiesMixin):
             return defer.succeed(None)
         if self.stopped:
             return defer.succeed(None)
-        log.msg("acquireLocks(build %s, locks %s)" % (self, self.locks))
+        log.msg("acquireLocks(build {}, locks {})".format(self, self.locks))
         for lock, access in self.locks:
             if not lock.isAvailable(self, access):
-                log.msg("Build %s waiting for lock %s" % (self, lock))
+                log.msg("Build {} waiting for lock {}".format(self, lock))
                 d = lock.waitUntilMaybeAvailable(self, access)
                 d.addCallback(self.acquireLocks)
                 self._acquiringLock = (lock, access, d)
@@ -454,7 +459,7 @@ class Build(properties.PropertiesMixin):
             count = self.stepnames[name]
             count += 1
             self.stepnames[name] = count
-            name = "%s_%d" % (step.name, count)
+            name = "{}_{}".format(step.name, count)
         else:
             self.stepnames[name] = 0
         step.name = name
@@ -465,7 +470,6 @@ class Build(properties.PropertiesMixin):
             step = factory.buildStep()
             step.setBuild(self)
             step.setWorker(self.workerforbuilder.worker)
-            self.setUniqueStepName(step)
             steps.append(step)
 
             if self.useProgress:
@@ -530,6 +534,7 @@ class Build(properties.PropertiesMixin):
         d.addBoth(self._flushProperties)
         d.addCallback(self._stepDone, s)
         d.addErrback(self.buildException)
+        return None
 
     @defer.inlineCallbacks
     def _flushProperties(self, results):
@@ -562,7 +567,7 @@ class Build(properties.PropertiesMixin):
         summary = yield step.getBuildResultSummary()
         if 'build' in summary:
             text = [summary['build']]
-        log.msg(" step '%s' complete: %s (%s)" % (step.name, statusToString(results), text))
+        log.msg(" step '{}' complete: {} ({})".format(step.name, statusToString(results), text))
         if text:
             self.text.extend(text)
             self.master.data.updates.setBuildStateString(self.buildid,
@@ -579,7 +584,7 @@ class Build(properties.PropertiesMixin):
         # the worker went away. There are several possible reasons for this,
         # and they aren't necessarily fatal. For now, kill the build, but
         # TODO: see if we can resume the build when it reconnects.
-        log.msg("%s.lostRemote" % self)
+        log.msg("{}.lostRemote".format(self))
         self.conn = None
         self.text = ["lost", "connection"]
         self.results = RETRY
@@ -593,7 +598,6 @@ class Build(properties.PropertiesMixin):
             if self._acquiringLock:
                 lock, access, d = self._acquiringLock
                 lock.stopWaitingUntilAvailable(self, access, d)
-                d.callback(None)
 
     def controlStopBuild(self, key, params):
         return self.stopBuild(**params)
@@ -606,7 +610,7 @@ class Build(properties.PropertiesMixin):
         # build as failed quickly rather than waiting for the worker's
         # timeout to kill it on its own.
 
-        log.msg(" %s: stopping build: %s %d" % (self, reason, results))
+        log.msg(" {}: stopping build: {} {}".format(self, reason, results))
         if self.finished:
             return
         # TODO: include 'reason' in this point event
@@ -619,7 +623,6 @@ class Build(properties.PropertiesMixin):
         if self._acquiringLock:
             lock, access, d = self._acquiringLock
             lock.stopWaitingUntilAvailable(self, access, d)
-            d.callback(None)
 
     def allStepsDone(self):
         if self.results == FAILURE:
@@ -638,7 +641,7 @@ class Build(properties.PropertiesMixin):
         return self.buildFinished(text, self.results)
 
     def buildException(self, why):
-        log.msg("%s.buildException" % self)
+        log.msg("{}.buildException".format(self))
         log.err(why)
         # try to finish the build, but since we've already faced an exception,
         # this may not work well.
@@ -666,7 +669,7 @@ class Build(properties.PropertiesMixin):
                 self.subs.unsubscribe()
                 self.subs = None
                 self.conn = None
-            log.msg(" %s: build finished" % self)
+            log.msg(" {}: build finished".format(self))
             self.results = worst_status(self.results, results)
             self.build_status.setText(text)
             self.build_status.setResults(self.results)
@@ -700,7 +703,7 @@ class Build(properties.PropertiesMixin):
 
     def releaseLocks(self):
         if self.locks:
-            log.msg("releaseLocks(%s): %s" % (self, self.locks))
+            log.msg("releaseLocks({}): {}".format(self, self.locks))
 
         for lock, access in self.locks:
             if lock.isOwner(self, access):
@@ -753,6 +756,9 @@ class Build(properties.PropertiesMixin):
             ('builds', str(self.buildid), 'finished'),
             lambda: self.finished)
 
+    def getWorkerInfo(self):
+        return self.workerforbuilder.worker.worker_status.info
+
     # IBuildControl
 
     def getStatus(self):
